#include <Metro.h>                       // Segment motion timer
#include <MsTimer2.h>                    // Servo speed timer
#include <Servo.h>

#define calibrateServos false

const int stepInterval = 40;             // MsTimer2 interval (ms)
const byte stepAmount = 2;               // Degree spacing
const int seqInterval = 550;             // Metro timer interval (ms)
const char maxV = 45;                    // CCW deflection
const char minV = -45;                   // CW deflection

const char segments[] = {                // Segment array
  minV, minV, 0, maxV, maxV, 0,          //  (enough for 10 servos,
  minV, minV, 0, maxV, maxV, 0,          //  6-step sequence) 
  minV, minV, 0, maxV, maxV, 0 };

Metro serialMetro = Metro(seqInterval);  // Set up segment motion timer
Servo myServos[10];                      // Set up 10 servos

volatile byte seq = 0;                   // Current sequence (out of 6)
char servo[10];                          // Current position for 10 servos

// Main setup function
void setup() {
  myServos[0].attach(A1);   // segment 1 // Attach all 10 servos
  myServos[1].attach(A2);   // segment 2
  myServos[2].attach(A3);   // segment 3
  myServos[3].attach(6);    // segment 4
  myServos[4].attach(5);    // segment 5
  myServos[5].attach(10);   // segment 6
  myServos[6].attach(12);   // segment 7
  myServos[7].attach(9);    // segment 8
  myServos[8].attach(8);    // segment 9 
  myServos[9].attach(7);    // segment 10 (head)

  // Either calibrate servos, or run motion  
  if (!calibrateServos) {
    // Bind MsTimer2 to 'moveInterval' function at stepInterval
    MsTimer2::set(stepInterval, moveInterval);  
    MsTimer2::start();
  } else {
    for (int i = 0; i < 10; i++) {
      myServos[i].write(90);
    }
  }
}

// Main repeating loop function
void loop() {
  if (!calibrateServos) {            // Not in calibrate mode
    if (serialMetro.check() == 1)    // If Metro timer has elapsed
      moveSegment();
  }
}

// Increment through 6 segment states
void moveSegment() {
  if (seq < 5) {   // If current segment < 5
    seq++;         //   go to next 
  } else {         // Current segment > 5
    seq = 0;       //   reset to 0
  }
}

// Set slow speed of servo; iterate through all 10
void moveInterval() {
  for (int i = 0; i < 10; i++) {         // Iterate for 10 servos
    if (segments[seq+i] > servo[i])      // Increase CCW to setpoint
      servo[i]+=stepAmount;
    else if (segments[seq+i] < servo[i]) // Decrease CW to setpoint
      servo[i]-=stepAmount;
    else servo[i] = segments[seq+i];

    myServos[i].write(servo[i]+90);      // Normalize for 0-180 degrees
  }
}

